@@ -220,7 +220,7 @@ PyAVDisposableReader = PyAVReader
 
 
 def create_stream(
-    container,
+    container: av.container.OutputContainer,
     codec_name: str,
     rate: int | fractions.Fraction,
     width: int,
@@ -230,12 +230,15 @@ def create_stream(
     time_base: fractions.Fraction,
     options: dict,
 ):
-    stream = container.add_stream(codec_name=codec_name, rate=rate)
-    stream.width = width
-    stream.height = height
-    stream.pix_fmt = pix_fmt
-    stream.bit_rate = bit_rate
-    stream.time_base = time_base
+    stream = container.add_stream(
+        codec_name=codec_name,
+        rate=rate,
+        width=width,
+        height=height,
+        pix_fmt=pix_fmt,
+        bit_rate=bit_rate,
+        time_base=time_base,
+    )
     stream.options = options
     return stream
 
@@ -246,10 +249,10 @@ class PyAVWriter(PyAVInterface):
     def __init__(
         self,
         path: str | Path | io.IOBase | None,
-        width: int = None,
-        height: int = None,
         fps: fractions.Fraction = None,
         *,
+        width: int = 640,
+        height: int = 480,
         codec_name="libvpx-vp9",
         pix_fmt="yuva420p",
         buffer_size=32768,
@@ -262,6 +265,7 @@ class PyAVWriter(PyAVInterface):
         audio_bit_rate=192000,
         format=None,
         options={},
+        container_options={},
     ):
         super().__init__()
 
@@ -285,6 +289,7 @@ class PyAVWriter(PyAVInterface):
         self._width = width
         self._height = height
         self._fps = fps
+        self._codex_contexts: dict[av.VideoStream, av.VideoCodecContext] = {}
         self._codec_name = codec_name
         self._pix_fmt = pix_fmt
         self._buffer_size = buffer_size
@@ -297,6 +302,7 @@ class PyAVWriter(PyAVInterface):
         self._audio_bit_rate = audio_bit_rate
         self._format = format
         self._options = options
+        self._container_options = container_options
 
         self._container = None
         self._alpha_extractor = None
@@ -326,6 +332,7 @@ class PyAVWriter(PyAVInterface):
             buffer_size=self._buffer_size,
             format=self._format,
             options=self._options,
+            container_options=self._container_options,
         )
         streams = []
         if self._codec_name is not None:
@@ -398,6 +405,56 @@ class PyAVWriter(PyAVInterface):
         frame = self.array_to_frame(array)
         self.write_video_frame(frame)
 
+    def create_codec_context(self, stream: av.VideoStream):
+        stream_cc = stream.codec_context
+        cc = stream.codec.create("video")
+
+        cc.width = stream.width
+        cc.height = stream.height
+
+        cc.pix_fmt = stream_cc.pix_fmt
+        cc.bit_rate = stream_cc.bit_rate
+        cc.time_base = stream_cc.time_base
+        cc.color_primaries = stream_cc.color_primaries
+        cc.color_range = stream_cc.color_range
+        cc.color_trc = stream_cc.color_trc
+
+        cc.framerate = stream_cc.framerate
+        cc.gop_size = stream_cc.gop_size
+        cc.qmax = stream_cc.qmax
+        cc.qmin = stream_cc.qmin
+
+        cc.options = stream_cc.options
+
+        return cc
+
+    def _encode_video_frame(self, stream: av.VideoStream, frame: av.VideoFrame):
+        cc = self._codex_contexts.get(stream.index)
+        if cc is None:
+            stream.width = frame.width
+            stream.height = frame.height
+            cc = stream.codec_context
+            if cc.coded_width == cc.coded_height == 0:
+                pass
+            elif (cc.coded_width, cc.coded_height) == (frame.width, frame.height):
+                pass
+            else:
+                cc = self.create_codec_context(stream)
+
+            self._codex_contexts[stream.index] = cc
+        elif (stream.width, stream.height) != (frame.width, frame.height):
+            stream.width = frame.width
+            stream.height = frame.height
+            for packet in cc.encode():
+                packet.stream = stream
+                yield packet
+            cc = self.create_codec_context(stream)
+            self._codex_contexts[stream.index] = cc
+
+        for packet in cc.encode_lazy(frame):
+            packet.stream = stream
+            yield packet
+
     def encode_video_frame(self, frame: av.VideoFrame):
         frames = [frame]
         if self.alpha_extractor is not None:
@@ -406,9 +463,7 @@ class PyAVWriter(PyAVInterface):
         for stream, frame in zip(self.streams, frames):
             frame.time_base = TIME_BASE
             frame.pts = round(self.__frames / self.fps / TIME_BASE)
-            for packet in stream.encode_lazy(frame):
-                packet.stream = stream
-                yield packet
+            yield from self._encode_video_frame(stream, frame)
 
         self.__frames += 1
 
@@ -454,7 +509,9 @@ class PyAVWriter(PyAVInterface):
         if self.alpha_extractor is not None:
             self.alpha_extractor.close()
         for stream in self.streams:
-            self.container.mux(stream.encode())
+            cc = self._codex_contexts.get(stream.index)
+            if cc is not None:
+                self.container.mux(cc.encode())
         if self.audio_stream is not None:
             self.container.mux(self.audio_stream.encode())
 
@@ -764,12 +821,17 @@ class BaseOverlayer(Filter):
         self.y = y
         self.pix_fmt = pix_fmt
         self.mode = mode
+        self.lock = threading.Lock()
 
     def _rebuild_graph(
         self,
         width: int,
         height: int,
         pix_fmt: str,
+        x: int,
+        y: int,
+        bg_width: int,
+        bg_height: int,
     ):
         self.src = self.graph.add_buffer(
             width=width,
@@ -778,8 +840,8 @@ class BaseOverlayer(Filter):
             time_base=fractions.Fraction(1, 1000),
         )
         self.dst = self.graph.add_buffer(
-            width=self.background_image.width,
-            height=self.background_image.height,
+            width=bg_width,
+            height=bg_height,
             format=self.background_image.format.name,
             time_base=fractions.Fraction(1, 1000),
         )
@@ -787,7 +849,7 @@ class BaseOverlayer(Filter):
         pre_filtered = self.pre_overlay_filter(self.graph, self.src)
 
         overlay = self.graph.add(
-            "overlay", f"x={self.x}:y={self.y}:alpha={self.mode}:format=auto"
+            "overlay", f"x={x}:y={y}:alpha={self.mode}:format=auto"
         )
         self.dst.link_to(overlay, input_idx=0)
         pre_filtered.link_to(overlay, input_idx=1)
@@ -801,20 +863,26 @@ class BaseOverlayer(Filter):
         post_filtered.link_to(sink)
 
     def paste(self, frame: av.VideoFrame):
-        self.rebuild_graph(
-            width=frame.width, height=frame.height, pix_fmt=frame.format.name
-        )
-        frame.pts = None
-        self.src.push(frame)
-        self.dst.push(self.background_image)
-        ret = self.graph.pull()
-        ret.pts = None
-        return ret
+        with self.lock:
+            self.rebuild_graph(
+                width=frame.width,
+                height=frame.height,
+                pix_fmt=frame.format.name,
+                x=self.x,
+                y=self.y,
+                bg_width=self.width,
+                bg_height=self.height,
+            )
+            frame.pts = None
+            self.src.push(frame)
+            self.dst.push(self.background_image)
+            ret = self.graph.pull()
+            ret.pts = None
+            return ret
 
     def paste_video(self, iterator: list[av.VideoFrame]):
         for it in iterator:
             yield self.paste(it)
-        self.close()
 
     @property
     def width(self):
@@ -837,9 +905,14 @@ class Overlayer(BaseOverlayer):
         pix_fmt: str = "yuva420p",
         mode: str = "straight",
     ):
-        dst_size, background_image = get_dst_size(dst_size, background_image)
-        src_pos, src_size = get_src_size(left, top, height, dst_size, src_size)
-
+        self.background_origin = background_image
+        src_size, src_pos, background_image = self._set_size(
+            dst_size=dst_size,
+            src_size=src_size,
+            left=left,
+            top=top,
+            height=height,
+        )
         self.src_size = src_size
 
         super().__init__(
@@ -851,6 +924,46 @@ class Overlayer(BaseOverlayer):
             pre_overlay_filter=self._pre_overlay_filter,
         )
 
+    def _set_size(
+        self,
+        dst_size: tuple[int, int],
+        src_size: tuple[int, int],
+        left=0.0,
+        top=0.0,
+        height=1.0,
+    ):
+        dst_size, background_image = get_dst_size(dst_size, self.background_origin)
+        src_pos, src_size = get_src_size(left, top, height, dst_size, src_size)
+        return src_size, src_pos, background_image
+
+    def set_size(
+        self,
+        dst_size: tuple[int, int],
+        src_size: tuple[int, int],
+        background_image: np.ndarray = None,
+        left=0.0,
+        top=0.0,
+        height=1.0,
+    ):
+        with self.lock:
+            if background_image is not None:
+                self.background_origin = background_image
+
+            src_size, src_pos, background_image = self._set_size(
+                dst_size=dst_size,
+                src_size=src_size,
+                left=left,
+                top=top,
+                height=height,
+            )
+            self.x = src_pos[0]
+            self.y = src_pos[1]
+            self.background_image = av.VideoFrame.from_ndarray(
+                background_image,
+                format="rgb24" if background_image.shape[-1] == 3 else "rgba",
+            )
+            self.src_size = src_size
+
     def _pre_overlay_filter(
         self, graph: av.filter.Graph, context: av.filter.context.FilterContext
     ) -> av.filter.context.FilterContext:
@@ -913,3 +1026,51 @@ class AsyncEncoder:
             return ret
         else:
             raise StopAsyncIteration
+
+
+class VideoEncoder:
+    def __init__(
+        self, *, stream: av.VideoStream, codec_context: av.VideoCodecContext, **kwargs
+    ):
+        for key, value in kwargs.items():
+            # if value is not None:
+            setattr(codec_context, key, value)
+
+        self.stream = stream
+        self.codec_context = codec_context
+
+    @classmethod
+    def from_stream(cls, stream: av.VideoStream, width: int, height: int, **kwargs):
+        stream_cc = stream.codec_context
+        cc = stream.codec.create("video")
+
+        _kwargs = {
+            "pix_fmt": stream_cc.pix_fmt,
+            "bit_rate": stream_cc.bit_rate,
+            "time_base": stream_cc.time_base,
+            "color_primaries": stream_cc.color_primaries,
+            "color_range": stream_cc.color_range,
+            "color_trc": stream_cc.color_trc,
+            "framerate": stream_cc.framerate,
+            "gop_size": stream_cc.gop_size,
+            "qmax": stream_cc.qmax,
+            "qmin": stream_cc.qmin,
+            "options": stream_cc.options,
+        }
+
+        for key, value in kwargs.items():
+            _kwargs[key] = value
+
+        return cls(
+            stream=stream,
+            codec_context=cc,
+            width=width,
+            height=height,
+            **_kwargs,
+        )
+
+    def encode(self, frame: av.VideoFrame):
+        packets = self.codec_context.encode(frame)
+        for packet in packets:
+            packet.stream = self.stream
+        return packets
